require_relative '../spec_helper'

describe ReplaceArithmeticOnRangesAst do
  
  it "should replace arithmetic on ranges (e.g., 1/A1:B2) with individual calculations (i.e., [1/A1, 1/B1, 1/A2, 1/B2])" do

    r = ReplaceArithmeticOnRangesAst.new

    r.map([:arithmetic, [:number, 1], [:operator, :"/"], [:cell, :F198]]).should == [:arithmetic, [:number, 1], [:operator, :"/"], [:cell, :F198]] 
    r.map([:arithmetic, [:number, 1], [:operator, :"/"], [:array, [:row, [:cell, :F197]], [:row, [:cell, :F198]], [:row, [:cell, :F199]]]]).should == [:array, [:row, [:arithmetic, [:number, 1], [:operator, :"/"], [:cell, :F197]]], [:row, [:arithmetic, [:number, 1], [:operator, :"/"], [:cell, :F198]]], [:row, [:arithmetic, [:number, 1], [:operator, :"/"], [:cell, :F199]]]]
    r.map([:arithmetic, [:array, [:row, [:cell, :F197]], [:row, [:cell, :F198]], [:row, [:cell, :F199]]], [:operator, :+], [:number, 1]]).should == [:array, [:row, [:arithmetic, [:cell, :F197], [:operator, :+], [:number, 1]]], [:row, [:arithmetic, [:cell, :F198], [:operator, :+], [:number, 1]]], [:row, [:arithmetic, [:cell, :F199], [:operator, :+], [:number, 1]]]]
    r.map([:arithmetic, [:array, [:row, [:cell, :F197]], [:row, [:cell, :F198]], [:row, [:cell, :F199]]], [:operator, :"/"],  [:array, [:row, [:cell, :F197]], [:row, [:cell, :F198]], [:row, [:cell, :F199]]]]).should == [:array, [:row, [:arithmetic, [:cell, :F197], [:operator, :"/"], [:cell, :F197]]], [:row, [:arithmetic, [:cell, :F198], [:operator, :"/"], [:cell, :F198]]], [:row, [:arithmetic, [:cell, :F199], [:operator, :"/"], [:cell, :F199]]]] 
  end

  it "should also replace comparisons on range with individual calculations (e.g., A1=B1:10)" do
    input_ast = [:function, :SUM, [:comparison, [:cell, :A1], [:comparator, "="], [:array, [:row, [:cell, :B1]], [:row, [:cell, :B2]]]]]
    output_ast = [:function, :SUM, [:array, [:row, [:comparison, [:cell, :A1], [:comparator, "="], [:cell, :B1]]], [:row, [:comparison, [:cell, :A1], [:comparator, "="], [:cell, :B2]]]]]
    r = ReplaceArithmeticOnRangesAst.new
    r.map(input_ast).should == output_ast

  end


  it "should work in complex nested cases" do
    input_ast = [:function,
                 :SUM,
                 [:arithmetic,
                  [:arithmetic,
                   [:array,
                    [:row,
                     [:sheet_reference, :"COM.DMD", [:cell, :K28]],
                     [:sheet_reference, :"COM.DMD", [:cell, :L28]],
                     [:sheet_reference, :"COM.DMD", [:cell, :M28]],
                     [:sheet_reference, :"COM.DMD", [:cell, :N28]],
                     [:sheet_reference, :"COM.DMD", [:cell, :O28]],
                     [:sheet_reference, :"COM.DMD", [:cell, :P28]],
                     [:sheet_reference, :"COM.DMD", [:cell, :Q28]],
                     [:sheet_reference, :"COM.DMD", [:cell, :R28]],
                     [:sheet_reference, :"COM.DMD", [:cell, :S28]],
                     [:sheet_reference, :"COM.DMD", [:cell, :T28]],
                     [:sheet_reference, :"COM.DMD", [:cell, :U28]],
                     [:sheet_reference, :"COM.DMD", [:cell, :V28]],
                     [:sheet_reference, :"COM.DMD", [:cell, :W28]],
                     [:sheet_reference, :"COM.DMD", [:cell, :X28]],
                     [:sheet_reference, :"COM.DMD", [:cell, :Y28]],
                     [:sheet_reference, :"COM.DMD", [:cell, :Z28]],
                     [:sheet_reference, :"COM.DMD", [:cell, :AA28]],
                     [:sheet_reference, :"COM.DMD", [:cell, :AB28]],
                     [:sheet_reference, :"COM.DMD", [:cell, :AC28]],
                     [:sheet_reference, :"COM.DMD", [:cell, :AD28]],
                     [:sheet_reference, :"COM.DMD", [:cell, :AE28]],
                     [:sheet_reference, :"COM.DMD", [:cell, :AF28]],
                     [:sheet_reference, :"COM.DMD", [:cell, :AG28]],
                     [:sheet_reference, :"COM.DMD", [:cell, :AH28]],
                     [:sheet_reference, :"COM.DMD", [:cell, :AI28]],
                     [:sheet_reference, :"COM.DMD", [:cell, :AJ28]],
                     [:sheet_reference, :"COM.DMD", [:cell, :AK28]],
                     [:sheet_reference, :"COM.DMD", [:cell, :AL28]],
                     [:sheet_reference, :"COM.DMD", [:cell, :AM28]],
                     [:sheet_reference, :"COM.DMD", [:cell, :AN28]],
                     [:sheet_reference, :"COM.DMD", [:cell, :AO28]]]],
                   [:operator, :*],
                   [:comparison,
                    [:sheet_reference, :"COM.DMD", [:cell, :L6]],
                    [:comparator, :"="],
                    [:array,
                     [:row,
                      [:sheet_reference, :"COM.DMD", [:cell, :K6]],
                      [:sheet_reference, :"COM.DMD", [:cell, :L6]],
                      [:sheet_reference, :"COM.DMD", [:cell, :M6]],
                      [:sheet_reference, :"COM.DMD", [:cell, :N6]],
                      [:sheet_reference, :"COM.DMD", [:cell, :O6]],
                      [:sheet_reference, :"COM.DMD", [:cell, :P6]],
                      [:sheet_reference, :"COM.DMD", [:cell, :Q6]],
                      [:sheet_reference, :"COM.DMD", [:cell, :R6]],
                      [:sheet_reference, :"COM.DMD", [:cell, :S6]],
                      [:sheet_reference, :"COM.DMD", [:cell, :T6]],
                      [:sheet_reference, :"COM.DMD", [:cell, :U6]],
                      [:sheet_reference, :"COM.DMD", [:cell, :V6]],
                      [:sheet_reference, :"COM.DMD", [:cell, :W6]],
                      [:sheet_reference, :"COM.DMD", [:cell, :X6]],
                      [:sheet_reference, :"COM.DMD", [:cell, :Y6]],
                      [:sheet_reference, :"COM.DMD", [:cell, :Z6]],
                      [:sheet_reference, :"COM.DMD", [:cell, :AA6]],
                      [:sheet_reference, :"COM.DMD", [:cell, :AB6]],
                      [:sheet_reference, :"COM.DMD", [:cell, :AC6]],
                      [:sheet_reference, :"COM.DMD", [:cell, :AD6]],
                      [:sheet_reference, :"COM.DMD", [:cell, :AE6]],
                      [:sheet_reference, :"COM.DMD", [:cell, :AF6]],
                      [:sheet_reference, :"COM.DMD", [:cell, :AG6]],
                      [:sheet_reference, :"COM.DMD", [:cell, :AH6]],
                      [:sheet_reference, :"COM.DMD", [:cell, :AI6]],
                      [:sheet_reference, :"COM.DMD", [:cell, :AJ6]],
                      [:sheet_reference, :"COM.DMD", [:cell, :AK6]],
                      [:sheet_reference, :"COM.DMD", [:cell, :AL6]],
                      [:sheet_reference, :"COM.DMD", [:cell, :AM6]],
                      [:sheet_reference, :"COM.DMD", [:cell, :AN6]],
                      [:sheet_reference, :"COM.DMD", [:cell, :AO6]]]]]],
                  [:operator, :*],
                  [:comparison,
                   [:sheet_reference, :"COM.DMD", [:cell, :B30]],
                   [:comparator, :"="],
                   [:array,
                    [:row,
                     [:sheet_reference, :"COM.DMD", [:cell, :K9]],
                     [:sheet_reference, :"COM.DMD", [:cell, :L9]],
                     [:sheet_reference, :"COM.DMD", [:cell, :M9]],
                     [:sheet_reference, :"COM.DMD", [:cell, :N9]],
                     [:sheet_reference, :"COM.DMD", [:cell, :O9]],
                     [:sheet_reference, :"COM.DMD", [:cell, :P9]],
                     [:sheet_reference, :"COM.DMD", [:cell, :Q9]],
                     [:sheet_reference, :"COM.DMD", [:cell, :R9]],
                     [:sheet_reference, :"COM.DMD", [:cell, :S9]],
                     [:sheet_reference, :"COM.DMD", [:cell, :T9]],
                     [:sheet_reference, :"COM.DMD", [:cell, :U9]],
                     [:sheet_reference, :"COM.DMD", [:cell, :V9]],
                     [:sheet_reference, :"COM.DMD", [:cell, :W9]],
                     [:sheet_reference, :"COM.DMD", [:cell, :X9]],
                     [:sheet_reference, :"COM.DMD", [:cell, :Y9]],
                     [:sheet_reference, :"COM.DMD", [:cell, :Z9]],
                     [:sheet_reference, :"COM.DMD", [:cell, :AA9]],
                     [:sheet_reference, :"COM.DMD", [:cell, :AB9]],
                     [:sheet_reference, :"COM.DMD", [:cell, :AC9]],
                     [:sheet_reference, :"COM.DMD", [:cell, :AD9]],
                     [:sheet_reference, :"COM.DMD", [:cell, :AE9]],
                     [:sheet_reference, :"COM.DMD", [:cell, :AF9]],
                     [:sheet_reference, :"COM.DMD", [:cell, :AG9]],
                     [:sheet_reference, :"COM.DMD", [:cell, :AH9]],
                     [:sheet_reference, :"COM.DMD", [:cell, :AI9]],
                     [:sheet_reference, :"COM.DMD", [:cell, :AJ9]],
                     [:sheet_reference, :"COM.DMD", [:cell, :AK9]],
                     [:sheet_reference, :"COM.DMD", [:cell, :AL9]],
                     [:sheet_reference, :"COM.DMD", [:cell, :AM9]],
                     [:sheet_reference, :"COM.DMD", [:cell, :AN9]],
                     [:sheet_reference, :"COM.DMD", [:cell, :AO9]]]]]]]

    output_ast = [:function, :SUM, [:array, [:row, [:arithmetic, [:arithmetic, [:sheet_reference, :"COM.DMD", [:cell, :K28]], [:operator, :*], [:comparison, [:sheet_reference, :"COM.DMD", [:cell, :L6]], [:comparator, :"="], [:sheet_reference, :"COM.DMD", [:cell, :K6]]]], [:operator, :*], [:comparison, [:sheet_reference, :"COM.DMD", [:cell, :B30]], [:comparator, :"="], [:sheet_reference, :"COM.DMD", [:cell, :K9]]]], [:arithmetic, [:arithmetic, [:sheet_reference, :"COM.DMD", [:cell, :L28]], [:operator, :*], [:comparison, [:sheet_reference, :"COM.DMD", [:cell, :L6]], [:comparator, :"="], [:sheet_reference, :"COM.DMD", [:cell, :L6]]]], [:operator, :*], [:comparison, [:sheet_reference, :"COM.DMD", [:cell, :B30]], [:comparator, :"="], [:sheet_reference, :"COM.DMD", [:cell, :L9]]]], [:arithmetic, [:arithmetic, [:sheet_reference, :"COM.DMD", [:cell, :M28]], [:operator, :*], [:comparison, [:sheet_reference, :"COM.DMD", [:cell, :L6]], [:comparator, :"="], [:sheet_reference, :"COM.DMD", [:cell, :M6]]]], [:operator, :*], [:comparison, [:sheet_reference, :"COM.DMD", [:cell, :B30]], [:comparator, :"="], [:sheet_reference, :"COM.DMD", [:cell, :M9]]]], [:arithmetic, [:arithmetic, [:sheet_reference, :"COM.DMD", [:cell, :N28]], [:operator, :*], [:comparison, [:sheet_reference, :"COM.DMD", [:cell, :L6]], [:comparator, :"="], [:sheet_reference, :"COM.DMD", [:cell, :N6]]]], [:operator, :*], [:comparison, [:sheet_reference, :"COM.DMD", [:cell, :B30]], [:comparator, :"="], [:sheet_reference, :"COM.DMD", [:cell, :N9]]]], [:arithmetic, [:arithmetic, [:sheet_reference, :"COM.DMD", [:cell, :O28]], [:operator, :*], [:comparison, [:sheet_reference, :"COM.DMD", [:cell, :L6]], [:comparator, :"="], [:sheet_reference, :"COM.DMD", [:cell, :O6]]]], [:operator, :*], [:comparison, [:sheet_reference, :"COM.DMD", [:cell, :B30]], [:comparator, :"="], [:sheet_reference, :"COM.DMD", [:cell, :O9]]]], [:arithmetic, [:arithmetic, [:sheet_reference, :"COM.DMD", [:cell, :P28]], [:operator, :*], [:comparison, [:sheet_reference, :"COM.DMD", [:cell, :L6]], [:comparator, :"="], [:sheet_reference, :"COM.DMD", [:cell, :P6]]]], [:operator, :*], [:comparison, [:sheet_reference, :"COM.DMD", [:cell, :B30]], [:comparator, :"="], [:sheet_reference, :"COM.DMD", [:cell, :P9]]]], [:arithmetic, [:arithmetic, [:sheet_reference, :"COM.DMD", [:cell, :Q28]], [:operator, :*], [:comparison, [:sheet_reference, :"COM.DMD", [:cell, :L6]], [:comparator, :"="], [:sheet_reference, :"COM.DMD", [:cell, :Q6]]]], [:operator, :*], [:comparison, [:sheet_reference, :"COM.DMD", [:cell, :B30]], [:comparator, :"="], [:sheet_reference, :"COM.DMD", [:cell, :Q9]]]], [:arithmetic, [:arithmetic, [:sheet_reference, :"COM.DMD", [:cell, :R28]], [:operator, :*], [:comparison, [:sheet_reference, :"COM.DMD", [:cell, :L6]], [:comparator, :"="], [:sheet_reference, :"COM.DMD", [:cell, :R6]]]], [:operator, :*], [:comparison, [:sheet_reference, :"COM.DMD", [:cell, :B30]], [:comparator, :"="], [:sheet_reference, :"COM.DMD", [:cell, :R9]]]], [:arithmetic, [:arithmetic, [:sheet_reference, :"COM.DMD", [:cell, :S28]], [:operator, :*], [:comparison, [:sheet_reference, :"COM.DMD", [:cell, :L6]], [:comparator, :"="], [:sheet_reference, :"COM.DMD", [:cell, :S6]]]], [:operator, :*], [:comparison, [:sheet_reference, :"COM.DMD", [:cell, :B30]], [:comparator, :"="], [:sheet_reference, :"COM.DMD", [:cell, :S9]]]], [:arithmetic, [:arithmetic, [:sheet_reference, :"COM.DMD", [:cell, :T28]], [:operator, :*], [:comparison, [:sheet_reference, :"COM.DMD", [:cell, :L6]], [:comparator, :"="], [:sheet_reference, :"COM.DMD", [:cell, :T6]]]], [:operator, :*], [:comparison, [:sheet_reference, :"COM.DMD", [:cell, :B30]], [:comparator, :"="], [:sheet_reference, :"COM.DMD", [:cell, :T9]]]], [:arithmetic, [:arithmetic, [:sheet_reference, :"COM.DMD", [:cell, :U28]], [:operator, :*], [:comparison, [:sheet_reference, :"COM.DMD", [:cell, :L6]], [:comparator, :"="], [:sheet_reference, :"COM.DMD", [:cell, :U6]]]], [:operator, :*], [:comparison, [:sheet_reference, :"COM.DMD", [:cell, :B30]], [:comparator, :"="], [:sheet_reference, :"COM.DMD", [:cell, :U9]]]], [:arithmetic, [:arithmetic, [:sheet_reference, :"COM.DMD", [:cell, :V28]], [:operator, :*], [:comparison, [:sheet_reference, :"COM.DMD", [:cell, :L6]], [:comparator, :"="], [:sheet_reference, :"COM.DMD", [:cell, :V6]]]], [:operator, :*], [:comparison, [:sheet_reference, :"COM.DMD", [:cell, :B30]], [:comparator, :"="], [:sheet_reference, :"COM.DMD", [:cell, :V9]]]], [:arithmetic, [:arithmetic, [:sheet_reference, :"COM.DMD", [:cell, :W28]], [:operator, :*], [:comparison, [:sheet_reference, :"COM.DMD", [:cell, :L6]], [:comparator, :"="], [:sheet_reference, :"COM.DMD", [:cell, :W6]]]], [:operator, :*], [:comparison, [:sheet_reference, :"COM.DMD", [:cell, :B30]], [:comparator, :"="], [:sheet_reference, :"COM.DMD", [:cell, :W9]]]], [:arithmetic, [:arithmetic, [:sheet_reference, :"COM.DMD", [:cell, :X28]], [:operator, :*], [:comparison, [:sheet_reference, :"COM.DMD", [:cell, :L6]], [:comparator, :"="], [:sheet_reference, :"COM.DMD", [:cell, :X6]]]], [:operator, :*], [:comparison, [:sheet_reference, :"COM.DMD", [:cell, :B30]], [:comparator, :"="], [:sheet_reference, :"COM.DMD", [:cell, :X9]]]], [:arithmetic, [:arithmetic, [:sheet_reference, :"COM.DMD", [:cell, :Y28]], [:operator, :*], [:comparison, [:sheet_reference, :"COM.DMD", [:cell, :L6]], [:comparator, :"="], [:sheet_reference, :"COM.DMD", [:cell, :Y6]]]], [:operator, :*], [:comparison, [:sheet_reference, :"COM.DMD", [:cell, :B30]], [:comparator, :"="], [:sheet_reference, :"COM.DMD", [:cell, :Y9]]]], [:arithmetic, [:arithmetic, [:sheet_reference, :"COM.DMD", [:cell, :Z28]], [:operator, :*], [:comparison, [:sheet_reference, :"COM.DMD", [:cell, :L6]], [:comparator, :"="], [:sheet_reference, :"COM.DMD", [:cell, :Z6]]]], [:operator, :*], [:comparison, [:sheet_reference, :"COM.DMD", [:cell, :B30]], [:comparator, :"="], [:sheet_reference, :"COM.DMD", [:cell, :Z9]]]], [:arithmetic, [:arithmetic, [:sheet_reference, :"COM.DMD", [:cell, :AA28]], [:operator, :*], [:comparison, [:sheet_reference, :"COM.DMD", [:cell, :L6]], [:comparator, :"="], [:sheet_reference, :"COM.DMD", [:cell, :AA6]]]], [:operator, :*], [:comparison, [:sheet_reference, :"COM.DMD", [:cell, :B30]], [:comparator, :"="], [:sheet_reference, :"COM.DMD", [:cell, :AA9]]]], [:arithmetic, [:arithmetic, [:sheet_reference, :"COM.DMD", [:cell, :AB28]], [:operator, :*], [:comparison, [:sheet_reference, :"COM.DMD", [:cell, :L6]], [:comparator, :"="], [:sheet_reference, :"COM.DMD", [:cell, :AB6]]]], [:operator, :*], [:comparison, [:sheet_reference, :"COM.DMD", [:cell, :B30]], [:comparator, :"="], [:sheet_reference, :"COM.DMD", [:cell, :AB9]]]], [:arithmetic, [:arithmetic, [:sheet_reference, :"COM.DMD", [:cell, :AC28]], [:operator, :*], [:comparison, [:sheet_reference, :"COM.DMD", [:cell, :L6]], [:comparator, :"="], [:sheet_reference, :"COM.DMD", [:cell, :AC6]]]], [:operator, :*], [:comparison, [:sheet_reference, :"COM.DMD", [:cell, :B30]], [:comparator, :"="], [:sheet_reference, :"COM.DMD", [:cell, :AC9]]]], [:arithmetic, [:arithmetic, [:sheet_reference, :"COM.DMD", [:cell, :AD28]], [:operator, :*], [:comparison, [:sheet_reference, :"COM.DMD", [:cell, :L6]], [:comparator, :"="], [:sheet_reference, :"COM.DMD", [:cell, :AD6]]]], [:operator, :*], [:comparison, [:sheet_reference, :"COM.DMD", [:cell, :B30]], [:comparator, :"="], [:sheet_reference, :"COM.DMD", [:cell, :AD9]]]], [:arithmetic, [:arithmetic, [:sheet_reference, :"COM.DMD", [:cell, :AE28]], [:operator, :*], [:comparison, [:sheet_reference, :"COM.DMD", [:cell, :L6]], [:comparator, :"="], [:sheet_reference, :"COM.DMD", [:cell, :AE6]]]], [:operator, :*], [:comparison, [:sheet_reference, :"COM.DMD", [:cell, :B30]], [:comparator, :"="], [:sheet_reference, :"COM.DMD", [:cell, :AE9]]]], [:arithmetic, [:arithmetic, [:sheet_reference, :"COM.DMD", [:cell, :AF28]], [:operator, :*], [:comparison, [:sheet_reference, :"COM.DMD", [:cell, :L6]], [:comparator, :"="], [:sheet_reference, :"COM.DMD", [:cell, :AF6]]]], [:operator, :*], [:comparison, [:sheet_reference, :"COM.DMD", [:cell, :B30]], [:comparator, :"="], [:sheet_reference, :"COM.DMD", [:cell, :AF9]]]], [:arithmetic, [:arithmetic, [:sheet_reference, :"COM.DMD", [:cell, :AG28]], [:operator, :*], [:comparison, [:sheet_reference, :"COM.DMD", [:cell, :L6]], [:comparator, :"="], [:sheet_reference, :"COM.DMD", [:cell, :AG6]]]], [:operator, :*], [:comparison, [:sheet_reference, :"COM.DMD", [:cell, :B30]], [:comparator, :"="], [:sheet_reference, :"COM.DMD", [:cell, :AG9]]]], [:arithmetic, [:arithmetic, [:sheet_reference, :"COM.DMD", [:cell, :AH28]], [:operator, :*], [:comparison, [:sheet_reference, :"COM.DMD", [:cell, :L6]], [:comparator, :"="], [:sheet_reference, :"COM.DMD", [:cell, :AH6]]]], [:operator, :*], [:comparison, [:sheet_reference, :"COM.DMD", [:cell, :B30]], [:comparator, :"="], [:sheet_reference, :"COM.DMD", [:cell, :AH9]]]], [:arithmetic, [:arithmetic, [:sheet_reference, :"COM.DMD", [:cell, :AI28]], [:operator, :*], [:comparison, [:sheet_reference, :"COM.DMD", [:cell, :L6]], [:comparator, :"="], [:sheet_reference, :"COM.DMD", [:cell, :AI6]]]], [:operator, :*], [:comparison, [:sheet_reference, :"COM.DMD", [:cell, :B30]], [:comparator, :"="], [:sheet_reference, :"COM.DMD", [:cell, :AI9]]]], [:arithmetic, [:arithmetic, [:sheet_reference, :"COM.DMD", [:cell, :AJ28]], [:operator, :*], [:comparison, [:sheet_reference, :"COM.DMD", [:cell, :L6]], [:comparator, :"="], [:sheet_reference, :"COM.DMD", [:cell, :AJ6]]]], [:operator, :*], [:comparison, [:sheet_reference, :"COM.DMD", [:cell, :B30]], [:comparator, :"="], [:sheet_reference, :"COM.DMD", [:cell, :AJ9]]]], [:arithmetic, [:arithmetic, [:sheet_reference, :"COM.DMD", [:cell, :AK28]], [:operator, :*], [:comparison, [:sheet_reference, :"COM.DMD", [:cell, :L6]], [:comparator, :"="], [:sheet_reference, :"COM.DMD", [:cell, :AK6]]]], [:operator, :*], [:comparison, [:sheet_reference, :"COM.DMD", [:cell, :B30]], [:comparator, :"="], [:sheet_reference, :"COM.DMD", [:cell, :AK9]]]], [:arithmetic, [:arithmetic, [:sheet_reference, :"COM.DMD", [:cell, :AL28]], [:operator, :*], [:comparison, [:sheet_reference, :"COM.DMD", [:cell, :L6]], [:comparator, :"="], [:sheet_reference, :"COM.DMD", [:cell, :AL6]]]], [:operator, :*], [:comparison, [:sheet_reference, :"COM.DMD", [:cell, :B30]], [:comparator, :"="], [:sheet_reference, :"COM.DMD", [:cell, :AL9]]]], [:arithmetic, [:arithmetic, [:sheet_reference, :"COM.DMD", [:cell, :AM28]], [:operator, :*], [:comparison, [:sheet_reference, :"COM.DMD", [:cell, :L6]], [:comparator, :"="], [:sheet_reference, :"COM.DMD", [:cell, :AM6]]]], [:operator, :*], [:comparison, [:sheet_reference, :"COM.DMD", [:cell, :B30]], [:comparator, :"="], [:sheet_reference, :"COM.DMD", [:cell, :AM9]]]], [:arithmetic, [:arithmetic, [:sheet_reference, :"COM.DMD", [:cell, :AN28]], [:operator, :*], [:comparison, [:sheet_reference, :"COM.DMD", [:cell, :L6]], [:comparator, :"="], [:sheet_reference, :"COM.DMD", [:cell, :AN6]]]], [:operator, :*], [:comparison, [:sheet_reference, :"COM.DMD", [:cell, :B30]], [:comparator, :"="], [:sheet_reference, :"COM.DMD", [:cell, :AN9]]]], [:arithmetic, [:arithmetic, [:sheet_reference, :"COM.DMD", [:cell, :AO28]], [:operator, :*], [:comparison, [:sheet_reference, :"COM.DMD", [:cell, :L6]], [:comparator, :"="], [:sheet_reference, :"COM.DMD", [:cell, :AO6]]]], [:operator, :*], [:comparison, [:sheet_reference, :"COM.DMD", [:cell, :B30]], [:comparator, :"="], [:sheet_reference, :"COM.DMD", [:cell, :AO9]]]]]]] 

    r = ReplaceArithmeticOnRangesAst.new
    r.map(input_ast).should == output_ast
  end


  it "should work in this excessively arithmetic case" do 
    input_ast = [:arithmetic, [:arithmetic, [:arithmetic, [:arithmetic, [:arithmetic, [:arithmetic, [:arithmetic, [:arithmetic, [:arithmetic, [:arithmetic, [:arithmetic, [:arithmetic, [:arithmetic, [:arithmetic, [:arithmetic, [:arithmetic, [:arithmetic, [:arithmetic, [:arithmetic, [:arithmetic, [:arithmetic, [:arithmetic, [:arithmetic, [:arithmetic, [:arithmetic, [:arithmetic, [:arithmetic, [:arithmetic, [:arithmetic, [:arithmetic, [:arithmetic, [:arithmetic, [:arithmetic, [:arithmetic, [:arithmetic, [:arithmetic, [:arithmetic, [:arithmetic, [:arithmetic, [:arithmetic, [:arithmetic, [:arithmetic, [:arithmetic, [:arithmetic, [:sheet_reference, :WST, [:cell, :I483]], [:operator, :+], [:sheet_reference, :WST, [:cell, :L483]]], [:operator, :+], [:sheet_reference, :WST, [:cell, :O483]]], [:operator, :+], [:sheet_reference, :WST, [:cell, :R483]]], [:operator, :+], [:sheet_reference, :WST, [:cell, :U483]]], [:operator, :+], [:sheet_reference, :WST, [:cell, :X483]]], [:operator, :+], [:sheet_reference, :WST, [:cell, :AA483]]], [:operator, :+], [:sheet_reference, :WST, [:cell, :AD483]]], [:operator, :+], [:sheet_reference, :WST, [:cell, :AG483]]], [:operator, :+], [:sheet_reference, :WST, [:cell, :AJ483]]], [:operator, :+], [:sheet_reference, :WST, [:cell, :AM483]]], [:operator, :+], [:sheet_reference, :WST, [:cell, :AP483]]], [:operator, :+], [:sheet_reference, :WST, [:cell, :AS483]]], [:operator, :+], [:sheet_reference, :WST, [:cell, :AV483]]], [:operator, :+], [:sheet_reference, :WST, [:cell, :AY483]]], [:operator, :+], [:sheet_reference, :WST, [:cell, :BB483]]], [:operator, :+], [:sheet_reference, :WST, [:cell, :BE483]]], [:operator, :+], [:sheet_reference, :WST, [:cell, :BH483]]], [:operator, :+], [:sheet_reference, :WST, [:cell, :BK483]]], [:operator, :+], [:sheet_reference, :WST, [:cell, :BN483]]], [:operator, :+], [:sheet_reference, :WST, [:cell, :BQ483]]], [:operator, :+], [:sheet_reference, :WST, [:cell, :BT483]]], [:operator, :+], [:sheet_reference, :WST, [:cell, :BW483]]], [:operator, :+], [:sheet_reference, :WST, [:cell, :BZ483]]], [:operator, :+], [:sheet_reference, :WST, [:cell, :CC483]]], [:operator, :+], [:sheet_reference, :WST, [:cell, :CF483]]], [:operator, :+], [:sheet_reference, :WST, [:cell, :CI483]]], [:operator, :+], [:sheet_reference, :WST, [:cell, :CL483]]], [:operator, :+], [:sheet_reference, :WST, [:cell, :CO483]]], [:operator, :+], [:sheet_reference, :WST, [:cell, :CR483]]], [:operator, :+], [:sheet_reference, :WST, [:cell, :CU483]]], [:operator, :+], [:sheet_reference, :WST, [:cell, :CX483]]], [:operator, :+], [:sheet_reference, :WST, [:cell, :DA483]]], [:operator, :+], [:sheet_reference, :WST, [:cell, :DD483]]], [:operator, :+], [:sheet_reference, :WST, [:cell, :DG483]]], [:operator, :+], [:sheet_reference, :WST, [:cell, :DJ483]]], [:operator, :+], [:sheet_reference, :WST, [:cell, :DM483]]], [:operator, :+], [:sheet_reference, :WST, [:cell, :DP483]]], [:operator, :+], [:sheet_reference, :WST, [:cell, :DS483]]], [:operator, :+], [:sheet_reference, :WST, [:cell, :DV483]]], [:operator, :+], [:sheet_reference, :WST, [:cell, :DY483]]], [:operator, :+], [:sheet_reference, :WST, [:cell, :EB483]]], [:operator, :+], [:sheet_reference, :WST, [:cell, :EE483]]], [:operator, :+], [:sheet_reference, :WST, [:cell, :EH483]]], [:operator, :+], [:sheet_reference, :WST, [:cell, :EK483]]]
    output_ast = input_ast.dup


    r = ReplaceArithmeticOnRangesAst.new
    r.map(input_ast).should == output_ast
  end

  it "should work with prefixes" do
    i = [:prefix, "+", [:array, [:row, [:sheet_referenence, 'Sheet1', [:cell, 'A1']]], [:row, [:sheet_reference, 'Sheet1', [:cell, 'A2']]]]]
    e = [:array, [:row, [:prefix, '+', [:sheet_referenence, 'Sheet1', [:cell, 'A1']]]], [:row, [:prefix, '+', [:sheet_reference, 'Sheet1', [:cell, 'A2']]]]]
    r = ReplaceArithmeticOnRangesAst.new
    r.map(i).should == e
  end

  it "should work with prefixes inside operations" do
    i = [:arithmetic, [:prefix, "+", [:array, [:row, [:sheet_referenence, 'Sheet1', [:cell, 'A1']]], [:row, [:sheet_reference, 'Sheet1', [:cell, 'A2']]]]], [:operator, :-], [:number, 1.0]]
    e = [:array, [:row, [:arithmetic, [:prefix, '+', [:sheet_referenence, 'Sheet1', [:cell, 'A1']]], [:operator, :-], [:number, 1.0]]], [:row, [:arithmetic, [:prefix, '+', [:sheet_reference, 'Sheet1', [:cell, 'A2']]],[:operator, :-], [:number, 1.0]]]]
    r = ReplaceArithmeticOnRangesAst.new
    r.map(i).should == e
  end

  it "should work with nested functions" do
    input_ast = [:function,
                 :SUMPRODUCT,
                 [:arithmetic,
                  [:comparison,
                   [:function,
                    :RIGHT,
                    [:array,
                     [:row, [:sheet_reference, :"G.20 (data)", [:cell, :C850]]],
                     [:row, [:sheet_reference, :"G.20 (data)", [:cell, :C851]]],
                     [:row, [:sheet_reference, :"G.20 (data)", [:cell, :C852]]],
                     [:row, [:sheet_reference, :"G.20 (data)", [:cell, :C853]]],
                     [:row, [:sheet_reference, :"G.20 (data)", [:cell, :C854]]],
                     [:row, [:sheet_reference, :"G.20 (data)", [:cell, :C855]]],
                     [:row, [:sheet_reference, :"G.20 (data)", [:cell, :C856]]],
                     [:row, [:sheet_reference, :"G.20 (data)", [:cell, :C857]]],
                     [:row, [:sheet_reference, :"G.20 (data)", [:cell, :C858]]],
                     [:row, [:sheet_reference, :"G.20 (data)", [:cell, :C859]]],
                     [:row, [:sheet_reference, :"G.20 (data)", [:cell, :C860]]],
                     [:row, [:sheet_reference, :"G.20 (data)", [:cell, :C861]]]],
                    [:number, 5.0]],
                   [:comparator, :"="],
                   [:string, "urban"]],
                  [:operator, :*],
                  [:array,
                   [:row, [:sheet_reference, :"G.20 (data)", [:cell, :I850]]],
                   [:row, [:sheet_reference, :"G.20 (data)", [:cell, :I851]]],
                   [:row, [:sheet_reference, :"G.20 (data)", [:cell, :I852]]],
                   [:row, [:sheet_reference, :"G.20 (data)", [:cell, :I853]]],
                   [:row, [:sheet_reference, :"G.20 (data)", [:cell, :I854]]],
                   [:row, [:sheet_reference, :"G.20 (data)", [:cell, :I855]]],
                   [:row, [:sheet_reference, :"G.20 (data)", [:cell, :I856]]],
                   [:row, [:sheet_reference, :"G.20 (data)", [:cell, :I857]]],
                   [:row, [:sheet_reference, :"G.20 (data)", [:cell, :I858]]],
                   [:row, [:sheet_reference, :"G.20 (data)", [:cell, :I859]]],
                   [:row, [:sheet_reference, :"G.20 (data)", [:cell, :I860]]],
                   [:row, [:sheet_reference, :"G.20 (data)", [:cell, :I861]]]]]]

    output_ast = [:function,
                  :SUMPRODUCT,
                  [:array,
                   [:row,
                    [:arithmetic,
                     [:comparison,
                      [:function,
                       :RIGHT,
                       [:sheet_reference, :"G.20 (data)", [:cell, :C850]],
                       [:number, 5.0]],
                      [:comparator, :"="],
                      [:string, "urban"]],
                     [:operator, :*],
                     [:sheet_reference, :"G.20 (data)", [:cell, :I850]]]],
                   [:row,
                    [:arithmetic,
                     [:comparison,
                      [:function,
                       :RIGHT,
                       [:sheet_reference, :"G.20 (data)", [:cell, :C851]],
                       [:number, 5.0]],
                      [:comparator, :"="],
                      [:string, "urban"]],
                     [:operator, :*],
                     [:sheet_reference, :"G.20 (data)", [:cell, :I851]]]],
                   [:row,
                    [:arithmetic,
                     [:comparison,
                      [:function,
                       :RIGHT,
                       [:sheet_reference, :"G.20 (data)", [:cell, :C852]],
                       [:number, 5.0]],
                      [:comparator, :"="],
                      [:string, "urban"]],
                     [:operator, :*],
                     [:sheet_reference, :"G.20 (data)", [:cell, :I852]]]],
                   [:row,
                    [:arithmetic,
                     [:comparison,
                      [:function,
                       :RIGHT,
                       [:sheet_reference, :"G.20 (data)", [:cell, :C853]],
                       [:number, 5.0]],
                      [:comparator, :"="],
                      [:string, "urban"]],
                     [:operator, :*],
                     [:sheet_reference, :"G.20 (data)", [:cell, :I853]]]],
                   [:row,
                    [:arithmetic,
                     [:comparison,
                      [:function,
                       :RIGHT,
                       [:sheet_reference, :"G.20 (data)", [:cell, :C854]],
                       [:number, 5.0]],
                      [:comparator, :"="],
                      [:string, "urban"]],
                     [:operator, :*],
                     [:sheet_reference, :"G.20 (data)", [:cell, :I854]]]],
                   [:row,
                    [:arithmetic,
                     [:comparison,
                      [:function,
                       :RIGHT,
                       [:sheet_reference, :"G.20 (data)", [:cell, :C855]],
                       [:number, 5.0]],
                      [:comparator, :"="],
                      [:string, "urban"]],
                     [:operator, :*],
                     [:sheet_reference, :"G.20 (data)", [:cell, :I855]]]],
                   [:row,
                    [:arithmetic,
                     [:comparison,
                      [:function,
                       :RIGHT,
                       [:sheet_reference, :"G.20 (data)", [:cell, :C856]],
                       [:number, 5.0]],
                      [:comparator, :"="],
                      [:string, "urban"]],
                     [:operator, :*],
                     [:sheet_reference, :"G.20 (data)", [:cell, :I856]]]],
                   [:row,
                    [:arithmetic,
                     [:comparison,
                      [:function,
                       :RIGHT,
                       [:sheet_reference, :"G.20 (data)", [:cell, :C857]],
                       [:number, 5.0]],
                      [:comparator, :"="],
                      [:string, "urban"]],
                     [:operator, :*],
                     [:sheet_reference, :"G.20 (data)", [:cell, :I857]]]],
                   [:row,
                    [:arithmetic,
                     [:comparison,
                      [:function,
                       :RIGHT,
                       [:sheet_reference, :"G.20 (data)", [:cell, :C858]],
                       [:number, 5.0]],
                      [:comparator, :"="],
                      [:string, "urban"]],
                     [:operator, :*],
                     [:sheet_reference, :"G.20 (data)", [:cell, :I858]]]],
                   [:row,
                    [:arithmetic,
                     [:comparison,
                      [:function,
                       :RIGHT,
                       [:sheet_reference, :"G.20 (data)", [:cell, :C859]],
                       [:number, 5.0]],
                      [:comparator, :"="],
                      [:string, "urban"]],
                     [:operator, :*],
                     [:sheet_reference, :"G.20 (data)", [:cell, :I859]]]],
                   [:row,
                    [:arithmetic,
                     [:comparison,
                      [:function,
                       :RIGHT,
                       [:sheet_reference, :"G.20 (data)", [:cell, :C860]],
                       [:number, 5.0]],
                      [:comparator, :"="],
                      [:string, "urban"]],
                     [:operator, :*],
                     [:sheet_reference, :"G.20 (data)", [:cell, :I860]]]],
                   [:row,
                    [:arithmetic,
                     [:comparison,
                      [:function,
                       :RIGHT,
                       [:sheet_reference, :"G.20 (data)", [:cell, :C861]],
                       [:number, 5.0]],
                      [:comparator, :"="],
                      [:string, "urban"]],
                     [:operator, :*],
                     [:sheet_reference, :"G.20 (data)", [:cell, :I861]]]]]]

    r = ReplaceArithmeticOnRangesAst.new
    output = r.map(input_ast)
    output.should == output_ast

  end

end
